function VBDAobj = VBDAfit_multi(y,type,mdist,family,p,k,S,nVB,VA,filename)
%-------------------------------------------------------------------------
%%% Input arguments:
%--- y:      Sample data matrix, each column correspond to different
%            variables.
%--- type:   String vector whose elements indicate whether columns of y are
%            discrete or continous. Set each element to 'discrete' or 
%            'continous'.
%--- mdist:  Marginal distribution of y (must be ordinal). Distribution 
%            object for which functions <cdf> and <icdf> can be evaluated.
%--- family: Bivariate copula family to use for pair-copulas.
%--- p:      Markov order of D-Vine copula
%--- k:      Number of factors in the covariance matrix of the normal
%            approximation.
%--- S:      Number of evaluations of the D-Vine copula used to compute the
%            gradient needed for VB estimation.
%--- nVB:    Number of VB steps used for inference.
%--- VA:     Variational Bayes approximation to the distribution of u
%-------------------------------------------------------------------------
%%% Output arguments: 
%--- VBDAobj.gamma_mean: Variational posterior mean of copula parameters.
%--- VBDAobj.gamma_sd:   Variational posterior standard deviation of copula
%                        parameters.
%--- VBDAobj.mu:         Mean vector of approximating density q, on the 
%                        transformed copula parameters.
%--- VBDAobj.B:          Factors of the covariance matrix of approximating 
%                        density q, on the transformed copula parameters 
%                        (Sigma = B*B'+D.^2).
%--- VBDAobj.D:          Idiosyncratic terms of the covariance matrix of 
%                        approximating density q, on the transformed copula 
%                        parameters (Sigma = B*B'+D.^2).
%--- VBDAobj.muz         If VA = 2, or VA = 3, is the mean of the variatio-
%                        nal approximation to z = Phi^(-1)(u).
%--- VBDAobj.logsigmaz   If VA = 2, is log(sigma) of the variatio-nal appr-
%                        oximation to z = Phi^(-1)(u).
%--- VBDAobj.C           If VA = 3, is the matrix for which Sigma^(-1)=CC',
%                        where Sigma = var(z).
%--- VBDAobj.lambda:     Matrix with alternative reparametrization of mu
%                        and Sigma
%--- VBDAobj.LB:         Lower Bound values at each VB step.
%--- VBDAobj.dist:       CDF object for the ordinal data in y.
%--- VBDAobj.y:          Sample data vector.
%--- VBDAobj.type:       String vector whose elements indicate whether 
%                        columns of y are discrete or continuous. Set each
%                        element to 'discrete' or 'contiunous'.
%--- VBDAobj.family:     Bivariate copula family to use for pair-copulas.
%--- VBDAobj.p:          Markov order of D-Vine copula
%--- VBDAobj.k:          Number of factors in the covariace matrix of the 
%                        normal approximation.
%--- VBDAobj.S:          Number of evaluations of the D-Vine copula used to 
%                        compute the gradient needed for VB estimation.
%--- VBDAobj.nVB:        Number of VB steps used for inference.
%--- VBDAobj.VA:        Number of VB steps used for inference.
%-------------------------------------------------------------------------
disp('----> VB estimation underway')
[ai,bi,n,LB,Copula_choice,ct,lambda,t,mu,Sigma,r,muz,logsigmaz,C,T,ait,bit,isDiscrete,B,D] = VBinivalM(y,type,mdist,family,p,k,S,nVB,VA);
ADA = [];
tic
while(t<nVB)
    t = t+1;
    [theta,gamma] = q_lambda(mu,Sigma,S,family,B,D);
    [uD,z] = q_u(ait,bit,muz,logsigmaz,C,S,VA);
    u = uDaibi2u(isDiscrete,ai,bi,uD);
    [g_lambda_L,loghhat,log_theta,log_q_lambda,grad_lambda] = gradient_computeM(theta,gamma,u,z,ai,bi,Copula_choice,lambda,k,p,VA,ait,bit,ct);
    ct = ct_constant(loghhat,log_theta,log_q_lambda,grad_lambda);
    [Change_delta,ADA] = ADADELTA(ADA,g_lambda_L);
    lambda = lambda + Change_delta;
    [mu,Sigma,B,D,muz,logsigmaz,C] = lambda2musigma(lambda,n,numel(ait),k,VA);
    LB(t) = mean(loghhat+log_theta-log_q_lambda);
    if mod(t,10)==0
        disp(['      VB step = ' mat2str(t) ', Number of factors = ' mat2str(k) ', Time = ' mat2str(round(toc*1000)/1000) ' sec, VA = ' mat2str(VA) '.'])
        plot(LB(1:t))
        drawnow
        tic
    end
end
[~,gamma] = q_lambda(mu,Sigma,10000,family,B,D);
Param = Gamma2Param(gamma,r,p);
%%% --Output
VBDAobj.gamma_mean = mean(Param,5);
VBDAobj.gamma_sd = std(Param,[],5);
VBDAobj.mu = mu;
VBDAobj.B = B;
VBDAobj.D = D;
VBDAobj.muz = muz;
VBDAobj.logsigmaz = logsigmaz;
VBDAobj.C = C;
VBDAobj.lambda = lambda;
VBDAobj.LB = LB;
VBDAobj.mdist = mdist;
VBDAobj.y = y;
VBDAobj.type = type;
VBDAobj.family = Copula_choice;
VBDAobj.p = p;
VBDAobj.k = k;
VBDAobj.S = S;
VBDAobj.nVB = nVB;
VBDAobj.VA = VA;
disp('----> VB estimation finished')
if nargin == 10 && ~isempty(filename)
    save(filename,'VBDAobj')
end
clear grad_lambda_logq                                                     %removes persistent variables
%%%----->  From here on, code is only concerned with printing output to
%%% matlab command window.
try                                                                        % Just in case this code generates error in future matlab releases.
    [~,gamma] = q_lambda(VBDAobj.mu,[],40000,VBDAobj.family,VBDAobj.B,VBDAobj.D);
    Param = Gamma2Param(gamma,size(VBDAobj.y,2),VBDAobj.p);
    numpar = size(VBDAobj.gamma_mean,4);
    tau = kendalltau(Param,VBDAobj.family);
    tau_mean = mean(tau,4);
    tau_q = quantile(tau,[0.05 0.95],4);
    Mean_par = reshape(VBDAobj.gamma_mean,size(VBDAobj.y,2)^2*(VBDAobj.p+1),numel(VBDAobj.gamma_mean)/(size(VBDAobj.y,2)^2*(VBDAobj.p+1)));
    Mean_par = round(Mean_par(Mean_par(:,1)~=0,:)*1000)/1000;
    tau_par =  reshape(tau_mean,size(VBDAobj.y,2)^2*(VBDAobj.p+1),1);
    tau_par =  round(tau_par(tau_par(:,1)~=0,:)*1000)/1000;
    tau_q_par =  reshape(tau_q,size(VBDAobj.y,2)^2*(VBDAobj.p+1),2);
    tau_q_par =  round(tau_q_par(tau_q_par(:,1)~=0,:)*1000)/1000;
    temp = [Mean_par tau_par tau_q_par];
    temp1 = num2str([repmat(1234567891,size(temp,1),1) temp ],7);
    s = 0;
    for l = 1:VBDAobj.p+1
        for i = 1:size(VBDAobj.y,2)
            for j = 1:size(VBDAobj.y,2)
                if ~(l==1 && i>=j)
                    s = s +1;
                    temp1(s,1:17) = ['theta_{' mat2str(i) ',' mat2str(j) '}^{(' mat2str(l-1) ')}'];
                end
            end
        end
    end
    disp('--------------------------------------------------------------------------------------------------------------------')
    disp(['----------------------------- Estimates of ' mat2str(numpar) ' parameters for each of the ' mat2str(size(temp1,1)) ' pair-copulas -----------------------------'])
    disp('--------------------------------------------------------------------------------------------------------------------')
    disp('                      Par1          Par2          Par3          Par4          Par5       Spearman      Spearman_5%  Spearman_95%')
    disp(temp1)
    disp('--------------------------------------------------------------------------------------------------------------------')
    disp('--------------------------------------------------------------------------------------------------------------------')
    disp('--------------------------------------------------------------------------------------------------------------------')
catch
    warning('Error displaying output results')
end

